using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Http;
using Oqtane.Enums;
using Oqtane.Infrastructure;
using Oqtane.Models;
using Oqtane.Security;
using Oqtane.Shared;
using SZUAbsolventenverein.Module.EventRegistration.Models;
using SZUAbsolventenverein.Module.EventRegistration.Repository;

namespace SZUAbsolventenverein.Module.EventRegistration.Services
{
    public class ServerEventRegistrationService : IEventRegistrationService
    {
        private readonly IEventRepository _EventRegistrationRepository;
        private readonly IUserPermissions _userPermissions;
        private readonly ILogManager _logger;
        private readonly IHttpContextAccessor _accessor;
        private readonly Alias _alias;

        public ServerEventRegistrationService(IEventRepository EventRegistrationRepository, IUserPermissions userPermissions, ITenantManager tenantManager, ILogManager logger, IHttpContextAccessor accessor)
        {
            _EventRegistrationRepository = EventRegistrationRepository;
            _userPermissions = userPermissions;
            _logger = logger;
            _accessor = accessor;
            _alias = tenantManager.GetAlias();
        }

        public Task<Event> AddEventAsync(Event NewEvent)
        {
            if (_userPermissions.IsAuthorized(_accessor.HttpContext.User, _alias.SiteId, EntityNames.Module, NewEvent.ModuleId, PermissionNames.Edit))
            {
                NewEvent = _EventRegistrationRepository.AddEventRegistration(NewEvent);
                _logger.Log(LogLevel.Information, this, LogFunction.Create, "EventRegistration Added {NewEvent}", NewEvent);
            }
            else
            {
                _logger.Log(LogLevel.Error, this, LogFunction.Security, "Unauthorized EventRegistration Add Attempt {NewEvent}", NewEvent);
                NewEvent = null;
            }
            return Task.FromResult(NewEvent);
        }

        public Task<Response> AddOrUpdateResponseAsync(int EventId, int ModuleId, bool ResponseType)
        {
            throw new System.NotImplementedException();
        }

        public Task DeleteEventAsync(int EventId, int ModuleId)
        {
            if (_userPermissions.IsAuthorized(_accessor.HttpContext.User, _alias.SiteId, EntityNames.Module, ModuleId, PermissionNames.Edit))
            {
                _EventRegistrationRepository.DeleteEventRegistration(EventId);
                _logger.Log(LogLevel.Information, this, LogFunction.Delete, "EventRegistration Deleted {EventId}", EventId);
            }
            else
            {
                _logger.Log(LogLevel.Error, this, LogFunction.Security, "Unauthorized EventRegistration Delete Attempt {EventId} {ModuleId}", EventId, ModuleId);
            }
            return Task.CompletedTask;
        }

        public Task<(Event, Response)> GetEventDetails(int EventId, int ModuleId)
        {
            throw new System.NotImplementedException();
        }

        public Task<List<Response>> GetEventResponses(int EventId, int ModuleId)
        {
            throw new System.NotImplementedException();
        }

        public Task<List<Event>> GetEventsAsync(int ModuleId)
        {
            if (_userPermissions.IsAuthorized(_accessor.HttpContext.User, _alias.SiteId, EntityNames.Module, ModuleId, PermissionNames.View))
            {
                return Task.FromResult(_EventRegistrationRepository.GetEventRegistrations(ModuleId).ToList());
            }
            else
            {
                _logger.Log(LogLevel.Error, this, LogFunction.Security, "Unauthorized EventRegistration Get Attempt {ModuleId}", ModuleId);
                return null;
            }
        }

        public Task<Event> UpdateEventAsync(Event NewEvent)
        {
            if (_userPermissions.IsAuthorized(_accessor.HttpContext.User, _alias.SiteId, EntityNames.Module, NewEvent.ModuleId, PermissionNames.Edit))
            {
                NewEvent = _EventRegistrationRepository.UpdateEventRegistration(NewEvent);
                _logger.Log(LogLevel.Information, this, LogFunction.Update, "EventRegistration Updated {NewEvent}", NewEvent);
            }
            else
            {
                _logger.Log(LogLevel.Error, this, LogFunction.Security, "Unauthorized EventRegistration Update Attempt {NewEvent}", NewEvent);
                NewEvent = null;
            }
            return Task.FromResult(NewEvent);
        }

        /*

        public Task<Models.Event> GetEventRegistrationAsync(int EventRegistrationId, int ModuleId)
        {
            if (_userPermissions.IsAuthorized(_accessor.HttpContext.User, _alias.SiteId, EntityNames.Module, ModuleId, PermissionNames.View))
            {
                return Task.FromResult(_EventRegistrationRepository.GetEventRegistration(EventRegistrationId));
            }
            else
            {
                _logger.Log(LogLevel.Error, this, LogFunction.Security, "Unauthorized EventRegistration Get Attempt {EventRegistrationId} {ModuleId}", EventRegistrationId, ModuleId);
                return null;
            }
        }

        public Task<Models.Event> AddEventRegistrationAsync(Models.Event EventRegistration)
        {
            if (_userPermissions.IsAuthorized(_accessor.HttpContext.User, _alias.SiteId, EntityNames.Module, EventRegistration.ModuleId, PermissionNames.Edit))
            {
                EventRegistration = _EventRegistrationRepository.AddEventRegistration(EventRegistration);
                _logger.Log(LogLevel.Information, this, LogFunction.Create, "EventRegistration Added {EventRegistration}", EventRegistration);
            }
            else
            {
                _logger.Log(LogLevel.Error, this, LogFunction.Security, "Unauthorized EventRegistration Add Attempt {EventRegistration}", EventRegistration);
                EventRegistration = null;
            }
            return Task.FromResult(EventRegistration);
        }

        public Task<Models.Event> UpdateEventRegistrationAsync(Models.Event EventRegistration)
        {
            if (_userPermissions.IsAuthorized(_accessor.HttpContext.User, _alias.SiteId, EntityNames.Module, EventRegistration.ModuleId, PermissionNames.Edit))
            {
                EventRegistration = _EventRegistrationRepository.UpdateEventRegistration(EventRegistration);
                _logger.Log(LogLevel.Information, this, LogFunction.Update, "EventRegistration Updated {EventRegistration}", EventRegistration);
            }
            else
            {
                _logger.Log(LogLevel.Error, this, LogFunction.Security, "Unauthorized EventRegistration Update Attempt {EventRegistration}", EventRegistration);
                EventRegistration = null;
            }
            return Task.FromResult(EventRegistration);
        }

        }*/
    }
}
